package main

import (
	"context"
	"os"
	"os/signal"
	"syscall"
	"time"

	"git.loafle.net/commons/configuration-go"
	cdr "git.loafle.net/commons/di-go/registry"
	"git.loafle.net/commons/logging-go"
	occp "git.loafle.net/overflow/commons-go/config/probe"
	occi "git.loafle.net/overflow/commons-go/core/interfaces"
	"git.loafle.net/overflow/probe/auth"
	"git.loafle.net/overflow/probe/config"
	"git.loafle.net/overflow/probe/probe"
)

func init() {
	logging.InitializeLogger(config.ProbeLoggingConfigFilePath())
}

func main() {
	_config := &config.Config{}
	if err := configuration.Load(_config, config.ProbeConfigFilePath()); nil != err {
		logging.Logger().Panic(err)
	}

	cdr.RegisterResource(config.ConfigKey, _config)

	var instance interface{}

	go func() {
		if occp.ProbeStateTypeNotAuthorized == _config.Probe.State() {
			instance = &auth.Authenticator{}
			doneChan, err := instance.(occi.EndableStarter).EndableStart()
			if nil != err {
				logging.Logger().Error(err)
				os.Exit(1)
			}
			var ok bool
			err, ok = <-doneChan
			if !ok {
				return
			}
			if nil != err {
				logging.Logger().Error(err)
				os.Exit(1)
			}
			if err := instance.(occi.Stopper).Stop(context.Background()); err != nil {
				logging.Logger().Errorf("error: %v", err)
			}
		}

		instance = &probe.Probe{}
		doneChan, err := instance.(occi.EndableStarter).EndableStart()
		if nil != err {
			logging.Logger().Error(err)
			os.Exit(1)
		}
		var ok bool
		err, ok = <-doneChan
		if !ok {
			return
		}
		if nil != err {
			logging.Logger().Error(err)
			os.Exit(1)
		}
		if err := instance.(occi.Stopper).Stop(context.Background()); err != nil {
			logging.Logger().Errorf("error: %v", err)
		}
	}()

	interrupt := make(chan os.Signal, 1)
	signal.Notify(interrupt,
		syscall.SIGKILL,
		syscall.SIGSTOP,
		syscall.SIGHUP,
		syscall.SIGINT,
		syscall.SIGTERM,
		syscall.SIGQUIT)

	<-interrupt
	ctx, cancel := context.WithTimeout(context.Background(), 5*time.Second)
	defer cancel()
	if err := instance.(occi.Stopper).Stop(ctx); err != nil {
		logging.Logger().Errorf("error: %v", err)
	}
}
